package be.mygod.vpnhotspot.net

import android.content.Context
import android.net.ConnectivityManager
import android.net.Network
import android.net.NetworkCapabilities
import android.net.NetworkRequest
import be.mygod.vpnhotspot.App.Companion.app
import be.mygod.vpnhotspot.debugLog

object VpnMonitor : ConnectivityManager.NetworkCallback() {
    interface Callback {
        fun onAvailable(ifname: String)
        fun onLost(ifname: String)
    }

    private const val TAG = "VpnMonitor"

    private val manager = app.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
    private val request = NetworkRequest.Builder()
            .addTransportType(NetworkCapabilities.TRANSPORT_VPN)
            .removeCapability(NetworkCapabilities.NET_CAPABILITY_NOT_VPN)
            .build()
    private val callbacks = HashSet<Callback>()
    private var registered = false

    /**
     * Obtaining ifname in onLost doesn't work so we need to cache it in onAvailable.
     */
    private val available = HashMap<Network, String>()
    override fun onAvailable(network: Network) {
        val ifname = manager.getLinkProperties(network)?.interfaceName ?: return
        if (available.put(network, ifname) != null) return
        debugLog(TAG, "onAvailable: $ifname")
        callbacks.forEach { it.onAvailable(ifname) }
    }

    override fun onLost(network: Network) {
        val ifname = available.remove(network) ?: return
        debugLog(TAG, "onLost: $ifname")
        callbacks.forEach { it.onLost(ifname) }
    }

    fun registerCallback(callback: Callback) {
        if (!callbacks.add(callback)) return
        if (registered) available.forEach { callback.onAvailable(it.value) } else {
            manager.registerNetworkCallback(request, this)
            registered = true
        }
    }
    fun unregisterCallback(callback: Callback) {
        if (!callbacks.remove(callback) || callbacks.isNotEmpty() || !registered) return
        manager.unregisterNetworkCallback(this)
        registered = false
        available.clear()
    }
}
